/*
 This file contains all the network transport routines 
 for BNews (c) Robin Taylor. 2000
 */
#include "BeNews.h"
#include "constants.h"
/* Defined in mian.cpp */

extern char NewsServer[256];
extern char General[256];
extern int Socket;
extern unsigned int NNTP_$len;
extern unsigned int NewsPort;
extern unsigned LastResponse;

const uint32 B_OPENFILE = 'Mofm';
const uint32 B_MROT13_REQUEST = 'Mr13';
const uint32 B_XNA_REQUESTED  = 'Mxna';
const uint32 B_GED_REQUESTED  = 'Mged';
const uint32 B_ADS_REQUESTED  = 'Masd';
const uint32 B_POST_REQUESTED = 'Mpst';
const uint32 B_PBIN_REQUESTED = 'Mpbf';
const uint32 B_QUEU_REQUESTED = 'Mque';
const uint32 B_QRES_REQUESTED = 'Mqqq';
const uint32 B_CHECK_REQUESTED = 'Mchk';
const uint32 B_SIG_REQUESTED = 'Msig';
const uint32 B_FREEQ_REQUESTED = 'Mfqm';
const uint32 B_SCHK_REQUESTED = 'Mcks';
const uint32 B_LOOKUP_REQUESTED = 'Mlup';
const uint32 B_REPLACE_REQUESTED = 'Mrrq';
const uint32 B_ADDLEX_REQUESTED = 'Malx';
const uint32 B_WORD_CHANGE = 'Mwcr';
/* Opens a network Socket for the news port */
bool BeNewsWin::GetaSocket()
{
 int	s = -1;
 //long host_addr;
 struct	sockaddr_in sin;
 struct	hostent  *hp; //*gethostbyname (),

 memset((char *) &sin,'\0', sizeof (sin));

 // Create the socket. if >0 can't get a socked so quit, bad...
 if ((s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) return(false);
 
 // Set the address format for the imminent bind.
 //AF_INET ("Address Format: InterNET")
 sin.sin_family = AF_INET;
 //set port to 119 for net news translated to network byte order.
 sin.sin_port = htons (NewsPort);
 //first checks is the mahine is already a number or a numeric IP add string    
 if (!isdigit(NewsServer) || (long)(sin.sin_addr.s_addr = inet_addr (NewsServer)) == -1)
    {//no then it's a name and get host info incluning all IP address
   	   if((hp = gethostbyname (NewsServer)) == NULL)
		  {	//close socket and return
			closesocket(s);
			return (false);
		    }//As a convenience, the global h_addr constant is a fake field
		     //that points to the first item in the h_addr_list field.
			memcpy((char *) &sin.sin_addr, hp->h_addr, hp->h_length);
	 }
 	// And then connect 
	if (connect (s, (struct sockaddr *)&sin, sizeof(sin)) < 0)	
  	   { //close socket and return
		 closesocket(s);
		 return (false);
		 }
	    
//assign Socket
Socket=s;
return(true);
}

void BeNewsWin::NQuit(void)
{	

//if(BeAppQuitLock) return;

//BNews closing say goodbye....
	if(Socket>-1)
      {
       register char *cp, *tp;
	   register int num_bytes = 0;
       register unsigned int bytes_rcvd = 0;
       int blocking, result;
       extern int errno;

       //Initialise pointer and return/errno
       cp = NTTP_Line;
       *tp=*cp=(char)NULL;
       result = errno = B_OK;
             
       // Set sockets to none block 
       blocking = 1; // 1 non-blocking 0 blocking 
       setsockopt(Socket, SOL_SOCKET, SO_NONBLOCK, &blocking, sizeof(int));
       //Send quit but if the socket is dead, it should break the loop
       //give the news server a chance to say goodbye!! :)
       send(Socket, "QUIT\r\n", 6, 0);
       Ns->NSStat(0,"QUIT");
       //delay 30ms so we can have something to read
       snooze(300000);
       //Set the blocking variable to block for reset
       // as the longer we wait to read the better, I know only a couple of clock cycles...
       blocking=0;
       while(result == B_OK || result == EWOULDBLOCK)
          {  //Read 2 bytes.
           num_bytes = recv(Socket, cp, 1, 0);
      	   //Update the byte count
           bytes_rcvd++;
           //Break if \n is the last posible character
           //or a \r was returned previously
           if(*tp=='\r' && *cp=='\n') break;
           //Break on max bytes read defined in RFC-977
           if(bytes_rcvd == NNTP_$len) break;
           //If last chr or a \r switch to single byte reading
           //Some if we happen to have read one byte which ISN'T
           //Update the buffer pointer and do it again.
           tp=cp;
           cp++;
	       // If an error has occurred other than EWOULDBLOCK loop will break 
           result = errno;
           }
       if(tp!=NULL) *tp=(char)NULL;     
       //reassert blocking
       setsockopt(Socket, SOL_SOCKET, SO_NONBLOCK, &blocking, sizeof(int));
       //If we get a reply show it otherwiase continue to exit
       if(strlen(NTTP_Line)>3)
         {
          Ns->NSStat(1, NTTP_resp(atoi(NTTP_Line)));
          } 
	    //Close current Socket.
	    closesocket(Socket);
	    //make current socket invalid.
	    Socket=-1;
	   }
}
bool BeNewsWin::NCommand(char *string)
{	
	char be_str[NNTP_$len];
    int retry=0;

    //add the return+newline
	sprintf(be_str,"%s\r\n",string);
	//submit it to the server
RESEND:
    (void)send(Socket, &be_str, strlen(be_str), 0);
    //show the command,(in the dead time)
    if(NLoud) Ns->NSStat(0,string);
    //get the response code
    if(NLineReturn()<3)//repsonse code is 3 digits
 	  {   
       Ns->NSStat(1, "No data returned Retrying..");
       if(retry<1)
           {
	        retry++;
            goto RESEND;
            }
           if(!ReIntConnect()) return(false); 
        } else 
      {
      //Get the Response code
     if(strlen(NTTP_Line)<3) LastResponse = 999;
     else LastResponse = atoi(NTTP_Line);
     
     if(NLoud) Ns->NSStat(1, NTTP_resp(LastResponse));
     }
    if(LastResponse == 201) return(false);
	return(true);
}
/*
Re-Establishes a Lost connection.
*/
bool BeNewsWin::ReIntConnect(void)
{	
bool CFailed=false;
char Buffer[264];

//So this routine can be called to make a fresh connection
if(Socket>-1)
  {
	//Close current Socket.
	closesocket(Socket);
	//make current socket invalid.
	Socket=-1;
	CFailed=true;
    }
//get a Socket
if(GetaSocket()) Ns->NSStat(2,"Network - Ready");
 else {//Failed to get a socket
	   Ns->NSStat(2,"No Network.");
	   return(false);
	   }
//get the servers greeting
NLineReturn();
//Get the Response code
if(strlen(NTTP_Line)<3) LastResponse = 999;
   else LastResponse = atoi(NTTP_Line);
//Show it!
Ns->NSStat(1, NTTP_resp(LastResponse));
//Authenicate
if(Authenticate)
  {
   if(Authentication()) Ns->NSStat(2,"Authenicated.");
	else Ns->NSStat(2,"Authenication Failed.");
   }
//send MODE READER and use response as an INN check
if(NCommand("MODE READER") && LastResponse<500) INN=true;	
    else INN=false;
//If this is an attempt to recover a failed/dropped connection we should
//reselect the current group
if(CFailed)
  {//do it, I know these calls are recursive...
   sprintf(Buffer,"GROUP %s",CurrentGroupName);    
   NCommand(Buffer);
   }  	   
return(true);  
}

/*
Gets ONE line from the server <=  NNTP_$len
Returns the length of the object read.
Does NOT read past /r/n!
*/
unsigned int BeNewsWin::NLineReturn()
{
register char *cp, *tp;
register int num_bytes = 0;
register unsigned int bytes_rcvd = 0, rv = 2;
register unsigned int NNTP_$len_MinusOne;
int blocking, result;
extern int errno;

time_t timenow,timeloop;
//time out
time(&timenow);

//Initialise pointer and return/errno
cp = NTTP_Line;
result = errno = B_OK;
//So it's only avaluated once!
NNTP_$len_MinusOne = NNTP_$len-1;
// Set sockets to none block 
blocking = 1; // 1 non-blocking 0 blocking 
setsockopt(Socket, SOL_SOCKET, SO_NONBLOCK, &blocking, sizeof(int));

while((result == B_OK || result == EWOULDBLOCK))
    {  //Read 2 bytes.
       num_bytes = recv(Socket, cp, rv, 0);
       //did we get any?
       if (num_bytes == 2)
          {//copy the buffer pointer
           tp=cp;
           //Update the byte count
           bytes_rcvd += num_bytes;
           //Break if \n is the last posible character
           //or a \r was returned previously
           if(*tp=='\n' && rv==1) break;
           //Break on finding \r\n
           if(*tp++=='\r' && *tp=='\n') break;
           //Break on max bytes read defined in RFC-977
           if(bytes_rcvd == NNTP_$len) break;
           //If last chr or a \r switch to single byte reading
           if(bytes_rcvd == NNTP_$len_MinusOne || *tp=='\r') rv=1;
           //Update the buffer pointer and do it again.
           cp += num_bytes;
           	  } else  
           	        if (num_bytes == 1)
           	  		   {
           	            tp=cp;
           				//Update the byte count
                        bytes_rcvd++;
                        //Break if \n is the last posible character
                        //or a \r was returned previously
                        if(*tp=='\n' && rv==1) break;
                        //Break on max bytes read defined in RFC-977
                        if(bytes_rcvd == NNTP_$len) break;
                        //If last chr or a \r switch to single byte reading
                        //Some if we happen to have read one byte which ISN'T
                        // /r then the next time we'll still try for 2
                        if(bytes_rcvd == NNTP_$len_MinusOne || *tp=='\r') rv=1;
                        //Update the buffer pointer and do it again.
                        cp++;
          	             }
      				   
	 // If an error has occurred other than EWOULDBLOCK loop will break 
     result = errno;
    //We didn't get data thet time could have lost connection?
     if(num_bytes<1)
        { //if the loop isn't done after 60 seconds and theres a no bytes return quit loop
		 time(&timeloop);        
         if(difftime(timeloop, timenow)>60)
		   {
			Ns->NSStat(1, "Lost Connection!");
            break;
			}        
        }
    }

//make the line a valid string for the string ops.
cp++;
*cp=(char)NULL;
// Set socket back to blocking 
blocking = 0; 
setsockopt(Socket, SOL_SOCKET, SO_NONBLOCK, &blocking, sizeof(int));   
//Return Number of bytes recieved.
return(bytes_rcvd);
}


//-------------------------------------------------------------------//
//       All the posting code will live in this CPP file             //
//-------------------------------------------------------------------//
BeNewPost::BeNewPost(BeNewsRead* Bm,BeNewsWin* bn,bool newart)
	: BWindow(BRect(25, 60, 615, 470), "BNws-Poster", B_MODAL_WINDOW,
		B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS)
{	
    register char *cp,*tp;
    char *xp,*memoryalloc;
    off_t size;
    BFont tfont(be_fixed_font);
    rgb_color rgb;
    rgb_color rgb2;
    float XOrigin,YOrigin;
    BBox *B2;
    //No group so close the construct via B_Quit
	if(strlen(bn->CurrentGroupName)<1 && newart) PostMessage(B_QUIT_REQUESTED);
	//copy to class globals
	Bnews=bn;
	Bread=Bm;
	NArt=newart;
	//set to no binary
	SendBinary=false;
	//Make the RGB values for the text colour
	rgb.red=rgb.green=20;
	rgb.blue=150;
	rgb2.alpha=rgb.alpha=255;
	rgb2.red=rgb2.green=rgb2.blue=0;

    //calculate window size
	XOrigin=Bounds().right/2;
	YOrigin=Bounds().bottom/2;
	background = new BView(Bounds(), "view", B_FOLLOW_ALL, B_WILL_DRAW);
	//Set Background to BLACK
	background->SetViewColor(200, 200, 200, 255);
	//add 'background'
	AddChild(background);
    
 	//add the view for body
 	BodyView = new BTextView(BRect(5,82,571,365), "Body",BRect(6,0,569,283),&tfont, &rgb2 ,B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
 	background->AddChild(new BScrollView("Body", BodyView, B_FOLLOW_LEFT | B_FOLLOW_TOP, 0, true, true));
 	BodyView->SetStylable(true); 
    Uname = new BTextControl(BRect(5,5,215,25),"You","From","",0,B_FOLLOW_ALL, B_WILL_DRAW);
    Uname->SetDivider(35);
    Uname->SetEnabled(false);
    Subject = new BTextControl(BRect(230,5,584,25),"Subject","Subject","",0,B_FOLLOW_ALL, B_WILL_DRAW);
    Subject->SetDivider(40);
    Subject->SetEnabled(false);
    Groups= new BTextControl(BRect(5,31,584,51),"where","Groups","",0,B_FOLLOW_ALL, B_WILL_DRAW);
    Groups->SetDivider(35);
    //assign XNA's value here also
    Groups->SetEnabled(XNA=Ged=AdSig=false);
    XNoArchive = new BCheckBox(BRect(42,56,260,76),"XNO","X-No-Archive, (Suggests no archiving)",new BMessage(B_XNA_REQUESTED), B_FOLLOW_ALL, B_WILL_DRAW);
    AddSig = new BCheckBox(BRect(280,56,380,76),"EG","Add Signature",new BMessage(B_ADS_REQUESTED), B_FOLLOW_ALL, B_WILL_DRAW);
 	EditG = new BCheckBox(BRect(410,56,590,76),"EG","Switch to Group(s) Editable",new BMessage(B_GED_REQUESTED), B_FOLLOW_ALL, B_WILL_DRAW);
    background->AddChild(Uname);
    background->AddChild(Subject);
    background->AddChild(Groups);
    background->AddChild(EditG);
    background->AddChild(XNoArchive);
    background->AddChild(AddSig);
    
    Uname->SetText(Bnews->UserID);
    //Fills those nasty holes left in the scroll bars!
    B2= new BBox(BRect(572,365,586,381), NULL, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE_JUMP,B_NO_BORDER);
    background->AddChild(B2);
    B2->SetViewColor(191, 191, 191, 255);
	Quit = new BButton(BRect(482,383,585,409),"QButton","Close, Don't Send.",new BMessage(B_QRES_REQUESTED));
    Esig = new BButton(BRect(5,383,80,409),"NButton","Edit Sig.",new BMessage(B_SIG_REQUESTED));
   	ROT13= new BButton(BRect(80,383,155,409),"KButton","ROT13",new BMessage(B_MROT13_REQUEST));
    Chek = new BButton(BRect(155,383,230,409),"PButton","Spell Check",new BMessage(B_CHECK_REQUESTED));
 	Send = new BButton(BRect(240,383,356,409),"FButton","Post this article now!",new BMessage(B_POST_REQUESTED));
    SBin = new BButton(BRect(356,383,472,409),"BButton","Post a Binary now!",new BMessage(B_PBIN_REQUESTED));

	//Add the buttons
	background->AddChild(Esig);
 	background->AddChild(Chek);
 	background->AddChild(ROT13);
 	background->AddChild(SBin);
 	background->AddChild(Send);
  	background->AddChild(Quit);
   //if new article then subject is editable.. NewArt IS Bool!!

   Subject->SetEnabled(newart);
   if(newart) Groups->SetText(bn->CurrentGroupName);
   else //if not a new article 
     {//verify and set the subject eg Re: not RE: or re: 
      BDirectory dir("/boot/home/bnews");
      BFile Cf;
      char Newsgroups[256];
   
      char Buffer[2050];

      //The article you are responing to quote and atrribute get groups etc     
      if(dir.Contains("Currentarticle", B_ANY_NODE))
     	{
	     Cf= BFile("/boot/home/bnews/Currentarticle", B_READ_ONLY);
		//Lets get the article into memory!
		 Cf.GetSize(&size);
		 if((memoryalloc=(char*)malloc(size+2))!=NULL)
			{//We have the memory
			 cp=tp=memoryalloc;
			 xp=cp+size;
			 //read the whole file into memory
			 Cf.Read(cp,size);
			 //make the whole thing a string...
			 cp[size]=(char)NULL;
			 //get all fields..
			 //From
			 if((tp=strstr(cp,"From: "))!=NULL)
			   {
			    if((cp=strstr(tp," \n"))!=NULL)
			      {
			       *cp=(char)NULL;
			       tp+=6;
			       strcpy(Article.Name,tp);
			       *cp=' ';
			       }
			    }
			 cp=tp=memoryalloc;
			 // Newsgroups:
			 if((tp=strstr(cp,"Newsgroups: "))!=NULL)
			   {
			    if((cp=strstr(tp," \n"))!=NULL)
			      {
			       *cp=(char)NULL;
			       tp+=12;
			       strcpy(Newsgroups,tp);
			       *cp=' ';
			       }
			    }
			 cp=tp=memoryalloc;
			 //Subject (all fields that will go in the new header must had the extra [space] removed
			 if((tp=strstr(cp,"Subject: "))!=NULL)
			   {
			    if((cp=strstr(tp," \n"))!=NULL)
			      {
			       *cp=(char)NULL;
			       tp+=9;
			       strcpy(Article.Subject,tp);
			       *cp=' ';
			       }
			    }
			 cp=tp=memoryalloc;    
			 //Date
			 if((tp=strstr(cp,"Date: "))!=NULL)
			   {
			    if((cp=strstr(tp," \n"))!=NULL)
			      {
			       *cp=(char)NULL;
			       tp+=6;
			       strcpy(Article.Date,tp);
			       *cp=' ';
			       }
			    }
			 cp=tp=memoryalloc;
			 //Message ID
			 if((tp=strstr(cp,"Message-ID: "))!=NULL)
			   {
			    if((cp=strstr(tp," \n"))!=NULL)
			      {
			       *cp=(char)NULL;
			       tp+=12;
			       strcpy(Article.MessageID,tp);
			       *cp=' ';
			       }
			     }
			 cp=tp=memoryalloc;
			 //References
			 if((tp=strstr(cp,"References: "))!=NULL)
			   {
			    if((cp=strstr(tp," \n"))!=NULL)
			      {
			       *cp=(char)NULL;
			       tp+=12;
			       strcpy(Article.ReferenceID,tp);
			       *cp=' ';
			       }
			    }
			 cp=tp=memoryalloc;
			 
			 //create the Quote line the first line
             sprintf(Buffer,"On %s, %s wrote:\n\n",Article.Date,Article.Name);
             if(strlen(Buffer)>79)
              {//split a line greater than on line at the space nearest the end
              unsigned int i;
              for(i=79;i>0;)
                 {
                  if(Buffer[i]==' ')
                    {
                     Buffer[i]='\n';
                     break;
                    } 
                  i--;                     
                 }
               }
			 BodyView->SetText(Buffer);
			 //free that memory
			 size=strlen(Buffer);
			 if((cp=strstr(tp,"\n. \n"))!=NULL)   
			   {
			    char store,store2;
			    cp+=4;
			    while(tp<xp)
			     {
			      if((tp=strstr(cp," \n"))==NULL) break;
			      *tp='\n';
			      tp++;
			      *tp=(char)NULL;
			      //This code should NOT be necessary! but in the real world it is.
			      if(strlen(cp)>79)
					{//backup the char at cp[79] & cp[78]
					 int x9,x8=78;
					 //try not to split words
					 while(x8>60 && cp[x8]!=' ') x8--;
					 x9=x8;
					 x9++;
					 store=cp[x9];
					 store2=cp[x8];
					 //make cp a /n terminated string
					 cp[x9]=(char)NULL;
					 cp[x8]='\n';
					 //reset the end of the long line, which you can bet was 
					 //generated by a newbie with OE!
					 *tp='\n';
					 tp--;
					 *tp=' ';
					 //make the restored char the next to be looked at after the increment of tp
					 tp=cp+(x8-1);     
			         //make the string 
			          sprintf(Buffer,">%s",cp);
			         //restore the original chars
			         cp[x9]=store;
			         cp[x8]=store2;
			         } else sprintf(Buffer,">%s",cp);
				  BodyView->Insert((int32)size, Buffer, strlen(Buffer),NULL);
				  size+=strlen(Buffer);
				  tp++;
				  cp=tp;
			      }
			   	   
			   }	
			  //FREE up that memory!!!!! 
             free((void*)memoryalloc);
             //Write the groups
             Groups->SetText(Newsgroups);
             //MUST be 'Re: ' not 'RE: ' or any other nonesense
             //such as 'Re(2): ', 'Re:' or 'Sv: '
             cp=Article.Subject;
             if(cp[0]=='R' && cp[1]=='e' && cp[2]==':' && cp[3]==' ') strcpy(Buffer,Article.Subject);
             else //almost right just screwy case
             if((cp[0]=='R' || cp[0]=='r') && (cp[1]=='E' || cp[1]=='e') && cp[2]==':' && cp[3]==' ')
                {
                cp[0]='R';
                cp[1]='e';
                strcpy(Buffer,Article.Subject);
                }  
             else //some Re: and no space
             if((cp[0]=='R' || cp[0]=='r') && (cp[1]=='E' || cp[1]=='e') && cp[2]==':' && cp[3]!=' ')
                {
                cp+=3;
                strcpy(Buffer,"Re: ");
                strcat(Buffer,cp);
                }
             else //'Sv: ' never seen it but it's mentioned in a RFC as legacy so
             if(cp[0]=='S' && cp[1]=='v' && cp[2]==':' && cp[3]==' ')
               {
                cp[0]='R';
                cp[1]='e';
                strcpy(Buffer,Article.Subject);
                } 
             else//'Re(2): ' never seen it but it's mentioned in a RFC as legacy so
             if(cp[0]=='R' && cp[1]=='e' && cp[2]=='(' && cp[3]!='2' && cp[4]=='2' && cp[5]!=')' && cp[6]==':' && cp[7]==' ')
                {
                cp+=7;
                strcpy(Buffer,"Re: ");
                strcat(Buffer,cp);
                }   
             else  
                 {
                  strcpy(Buffer,"Re: ");
                  strcat(Buffer,cp);
                  }  

             Subject->SetText(Buffer);   
			}	 
		} else  PostMessage(B_QUIT_REQUESTED);  //can't get artilce to respone to quit!   
      }
//hide the first modal window which would stop the fileselect dialog for being used.
if(!NArt) Bread->Hide();
}

/////////////////////////////////////////////////////
// resend failed article                           //
/////////////////////////////////////////////////////
bool BeNewsWin::RecoverResend(void)
{
 BAlert *Alert;
 BDirectory dir("/boot/home/bnews");
 BEntry entry("/boot/home/bnews/NewArticle");
 BDirectory dirs("/boot/home/bnews/sent");
 char NewFile[255]="/boot/home/bnews/sent/";
 int number=0;
 BFile Cf;
 tm *timex;
 time_t timenow;
 unsigned int i,d,c;
 off_t size;
 time(&timenow);
 char *cp,*tp,*memoryalloc;
 char Buffer[2050],M_ID[256];
 
 if(dir.Contains("NewArticle", B_ANY_NODE))
   {//Open failed article,(or hacked headered article for CMSG
    Alert = new BAlert("RePost/Recovery","Do you wish to attempt to resend the last article you tried to send?","Yes, Resend.","No!",NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT); 
    Alert->SetShortcut(0, B_ESCAPE); 
    int32 button_index = Alert->Go();
    //User said no! then return.
    if(button_index==1)  return(false);
    Ns->NSStat(3, "Send. Unsent article.");  
    Cf= BFile("/boot/home/bnews/NewArticle", B_READ_WRITE);
    //get the article size
    Cf.GetSize(&size);
    //allocate memory for it
    if((memoryalloc=(char*)malloc(size+1))==NULL) 
	  {//Fail not enough memory
	   Cf.Unset();
	   return(false);
	   }
	cp=tp=memoryalloc;
	tp+=size;   
    //read it in
    Cf.Read(memoryalloc,size);
    //close it
    Cf.Unset();
    //delete it 
    //////////////////////////////////////////////
    //Create a new message ID of the new article//
    //////////////////////////////////////////////
    //Make Message ID     
 	strcpy(M_ID,"<BN.");	 
 	//Time Based component
 	timex = localtime(&timenow);
 	i=d=timex->tm_year;
 	d/=10;
 	d*=10;
 	i-=d;
 	//i= 0 to 9 yr + day/2 max=24 + 40h a-y
 	M_ID[4]=(char)(0x61+i+(timex->tm_mday/2));
 	//month A-J
 	M_ID[5]=(char)(0x41+timex->tm_mon);
 	//time based Hour
 	M_ID[6]=(char)(0x41+timex->tm_hour);
	//minute A-Z
 	M_ID[7]=(char)(0x41+(timex->tm_min/2.5));
 	//Make it a string again
 	M_ID[8]=(char)NULL;
 	//make sure we have user info
 	//SEED with minutes*100 + seconds +1, 100-590 + 1-60
 	i=((timex->tm_min*100)+(1+timex->tm_sec));
 	srand(i);
 	//User based component
 	i=strlen(UserID);
 	if(i>0)
  	  {
   		for(c=d=0;c<i;d+=UserID[c++]); 
   		d/=(1+i/3);
   		} else {
   				d=rand() & 0x160;
   				}		
	sprintf(Buffer,"%i",d);
 	//and a number sequence eg 312
 	strcat(M_ID,Buffer);	  
 	//Generate a large random alphanumeric ID and Add it!	
 	for(i=0;i<17;)
    	{
     	 //5Fh = 01011111 = 95 top upper case Alpha = 90
     	 d=rand() & 0x5f;
    	 if(d>0x40 && d<0x5b) 
        	{ //add the Alpha to the string and increment
          	 Buffer[i]=(char)d;
             i++;
          	 } else //try a Number the mix of alpha numerics will then be random
                 {   
              	  d=rand() & 0x3f;
     		  	  if(d>0x2f && d<0x3a) 
                    { //add the Numeric to the string and increment
                      Buffer[i]=(char)d;
          	   	      i++;
          		     } 
                 }
         }	  
 	//Add the L/R seperator      
 	Buffer[13]='@';      
 	//Make it a string      
 	Buffer[14]=(char)NULL;	  
 	//Add it to the LH side of the Message ID to complete it
 	strcat(M_ID,Buffer);
 	//copy server name
 	strcpy(Buffer,NewsServer);
 	//make it lowwer case
 	for(i=0;i<strlen(Buffer);Buffer[i]=(char)tolower(Buffer[i]),i++);
 	//remove news. if it exists
 	if((cp=strstr(Buffer,"news."))!=NULL)
   	  {
        //Eg. the news. is at the start
    	if(cp==Buffer) cp+=5;
      } else cp=Buffer;
 	//Make the ID!
 	strcat(M_ID,cp);
 	strcat(M_ID,">");
 	//Mesage ID
 	sprintf(Buffer,"Message-ID: %s\r\n",M_ID);
    //////////////////////////////////////////
    //Made new message ID                   //
    //////////////////////////////////////////
    //recreate a 'new' file
    //check for too many x-posts
    //check for x-posting..
    //Find the newsgroups line
    cp=memoryalloc;
    while(strncasecmp(cp,"Newsgroups:",11)!=0 && cp<tp) cp++;
    //check it found it rather than running out of file!
    if(strncasecmp(cp,"Newsgroups:",11)==0)
      {
       for(i=c=0; i<255;)
          {
           if(*cp++==',') c++;
           else if(*cp=='\n') break;
           }
	//Too many groups! quit resend
	    if(c>5)
	      {
	       free((void*)memoryalloc); 
	       Ns->NSStat(3, "Too many X-Posts!");  
	       return(false);
	       }
       } else
             {
	          free((void*)memoryalloc); 
	          Ns->NSStat(3, "Missing Newsgroups field!");  
	          return(false);
	          }
     //find the Message ID line and replace it!
     cp=memoryalloc;
     while(strncasecmp(cp,"Message-ID:",11)!=0 && cp<tp) cp++;
     //make sure we have it
     if(strncasecmp(cp,"Message-ID:",11)==0)
        {//Open a new file
          Cf= BFile("/boot/home/bnews/NewArticle", B_READ_WRITE | B_CREATE_FILE);
          //write the file up to that point
          Cf.Write(memoryalloc,(cp-memoryalloc));
          //write the new message ID
          Cf.Write(Buffer,strlen(Buffer));
          //write the rest
          //find the end of the original line
          if((cp=strchr(cp,'\n'))==NULL)
             {
              free((void*)memoryalloc); 
              return(false);
              }
          //make it the end of a string the size of which will be the code written
          *cp=(char)NULL;
          //first byte to write
          cp++;
          //total size-size written&skipped is whats left to write
          Cf.Write(cp,size-strlen(memoryalloc));
          //Free that memory
          free((void*)memoryalloc); 
           //seek the start of the file
	      Cf.Seek(0,SEEK_SET);
          //get its size
          Cf.GetSize(&size); 	   
   	      //allocate a buffer
          if((memoryalloc=(char*)malloc(size+1))==NULL) 
	  		{//Fail not enough memory - unlikely
	   		 Cf.Unset();
	         return(false);
	         }
          //Read the article into memory	  
	      Cf.Read(memoryalloc,size); 
          //close the file
          Cf.Unset();
          //Ask the server if it will accept the article 
          if(!NCommand("POST")) goto BADEND;
          //Command was ok so
          //340 send article to be posted. 
          if(LastResponse!=340) goto BADEND;
          //send article to the server
          (void)send(Socket, memoryalloc, size, 0);
          //get the response code
          if(NLineReturn()<3) goto BADEND;//repsonse code is 3 digits
          else LastResponse = atoi(NTTP_Line);
          Ns->NSStat(1, NTTP_resp(LastResponse));
          //v.important free the memory   *** 
          //Free that memory
          Ns->NSStat(3, "Article Sent."); 
          free((void*)memoryalloc); 
          //create sent folder if it does not exist
		  create_directory("/boot/home/bnews/sent/",0777);
          RETRY: 
          sprintf(Buffer,"bnSent-%i",number);
          if(dirs.Contains(Buffer, B_ANY_NODE))
	        {//get the first free filename name-number
	          number++;
	          goto RETRY;
	        } else
	              { //make a full path
			       strcat(NewFile,Buffer);
			       //copy the file form current article	      
			       entry.Rename(NewFile, true);
	               }
          return(true);
         } else
             {
	          free((void*)memoryalloc); 
	          Ns->NSStat(3, "Missing Message-ID field!");  
	          return(false);
	          }
	 BADEND:free((void*)memoryalloc); 
	        Ns->NSStat(3, "Re-send Failed");  
	        return(false);       
   }//no failed article
    else{
         Ns->NSStat(3, "Check - None Unsent."); 
         return(false);
         }

}

/////////////////////////////////////////////////////
//   Posts new news article last update from 0.6d  //
/////////////////////////////////////////////////////
void BeNewPost::PostNewArticle()
{
 BDirectory dir("/boot/home/bnews");
 BEntry entry("/boot/home/bnews/NewArticle");
 BDirectory dirs("/boot/home/bnews/sent");
 char NewFile[255]="/boot/home/bnews/sent/";
 int number=0;
 BFile Bf,Cf;
 tm *timex;
 time_t timenow;
 unsigned int i,c;
 off_t size,d;
 time(&timenow);
 char *cp,*tp,*memoryalloc;
 char NewsGroups[512],Buffer[2050],M_ID[256],Sig[405];
 unsigned int Linecount=0;
 BAlert *Alert; 
 
 if(dir.Contains("NewArticle", B_ANY_NODE))
   {
    //delete it 
     entry.Remove();
     }
  //get sig
	if(AdSig)
	  {
	   if(dir.Contains("Sigfile", B_ANY_NODE))
     	 {
          Cf= BFile("/boot/home/bnews/Sigfile", B_READ_ONLY);
          Cf.GetSize(&size);
	      if(size<401)
            {
			 tp=Sig;
             tp+=4;
             strcpy(Sig,"\n--\n");
             Cf.Read(tp,size);
             if(size<405) tp[size]=(char)NULL;
             while(Linecount<5)
                 {
                  if((tp=strchr(tp,'\n'))==NULL) break;
                  cp=++tp;
                  Linecount++;
                  }
             if(Linecount>0 && cp!=NULL) *cp=(char)NULL;
             //for the delimiter 
           	 Linecount+=2;
             }
           Cf.Unset(); 
           }      
	     }  
//add the line count of the inline binary
if(SendBinary) Linecount+=Binline;
 
 Cf= BFile("/boot/home/bnews/NewArticle", B_READ_WRITE | B_CREATE_FILE);
 //Write the HEADER line By Line
 //PATH
 Cf.Write("Path: not-for-mail\r\n",20);//8
 //FROM:
 sprintf(Buffer,"From: %s\r\n",Uname->Text());
 Cf.Write(Buffer,strlen(Buffer));
 //Newsgroups
 strcpy(NewsGroups,Groups->Text());
 //check for x-posting..
 for(i=c=0; i<strlen(NewsGroups);)
    {
     if(NewsGroups[i++]==',') c++;
     }
  if(c>5)
   {//No more than six groups!
   Alert = new BAlert("CrossPosting","Sorry, this article is crossposted to too many groups.","Cancel Post",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT); 
   Alert->SetShortcut(0, B_ESCAPE); 
   Alert->Go();
   return; 
   }
 sprintf(Buffer,"Newsgroups: %s\r\n",NewsGroups);
 Cf.Write(Buffer,strlen(Buffer));
 //Subject
 sprintf(Buffer,"Subject: %s\r\n",Subject->Text());
 Cf.Write(Buffer,strlen(Buffer));
 //Date  Wdy, DD Mon YY HH:MM:SS TIMEZONE (RFC-1036)
 timex = gmtime(&timenow); 
 //format the string
 strftime(Buffer,50,"Date: %a, %d %b %Y %H:%M:%S GMT\r\n",timex);
 Cf.Write(Buffer,strlen(Buffer));
 //Organization if there is one else don't bother it's optional why waste bandwidth..
 if(strlen(Bnews->UserOrg)>0)
   {
    sprintf(Buffer,"Organization: %s\r\n",Bnews->UserOrg);
 	Cf.Write(Buffer,strlen(Buffer));
 	}
 //Lines += becuase Sig may already of added lines!
 sprintf(Buffer,"Lines: %i\r\n",Linecount+=BodyView->CountLines());
 Cf.Write(Buffer,strlen(Buffer));
 //Distribution
 Cf.Write("Distribution: world\r\n",21);
 //Make Message ID     
 strcpy(M_ID,"<BN.");	 
 //Time Based component
 timex = localtime(&timenow);
 i=d=timex->tm_year;
 d/=10;
 d*=10;
 i-=d;
 //i= 0 to 9 yr + day/2 max=24 + 40h a-y
 M_ID[4]=(char)(0x61+i+(timex->tm_mday/2));
 //month A-J
 M_ID[5]=(char)(0x41+timex->tm_mon);
 //time based Hour
 M_ID[6]=(char)(0x41+timex->tm_hour);
//minute A-Z
 M_ID[7]=(char)(0x41+(timex->tm_min/2.5));
 //Make it a string again
 M_ID[8]=(char)NULL;
 //make sure we have user info
 //SEED with minutes*100 + seconds +1, 100-590 + 1-60
 i=((timex->tm_min*100)+(1+timex->tm_sec));
 srand(i);
 //User based component
 i=strlen(Bnews->UserID);
 if(i>0)
  {
   for(c=d=0;c<i;d+=Bnews->UserID[c++]); 
   d/=(1+i/3);
   } else {
   			d=rand() & 0x160;
   			}		
 sprintf(Buffer,"%i",(int)d);
 //and a number sequence eg 312
 strcat(M_ID,Buffer);	  
 //Generate a large random alphanumeric ID and Add it!	
 for(i=0;i<17;)
    {
     //5Fh = 01011111 = 95 top upper case Alpha = 90
     d=rand() & 0x5f;
     if(d>0x40 && d<0x5b) 
        { //add the Alpha to the string and increment
          Buffer[i]=(char)d;
          i++;
          } else //try a Number the mix of alpha numerics will then be random
             {   
              d=rand() & 0x3f;
     		  if(d>0x2f && d<0x3a) 
                 { //add the Numeric to the string and increment
                   Buffer[i]=(char)d;
          	   	   i++;
          		  } 
              }
    }	  
 //Add the L/R seperator      
 Buffer[13]='@';      
 //Make it a string      
 Buffer[14]=(char)NULL;	  
 //Add it to the LH side of the Message ID to complete it
 strcat(M_ID,Buffer);
 //copy server name
 strcpy(Buffer,NewsServer);
 //make it lowwer case
 for(i=0;i<strlen(Buffer);Buffer[i]=(char)tolower(Buffer[i]),i++);
 //remove news. if it exists
 if((cp=strstr(Buffer,"news."))!=NULL)
   {
    //Eg. the news. is at the start
    if(cp==Buffer) cp+=5;
    } else cp=Buffer;
 //Make the ID!
 strcat(M_ID,cp);
 strcat(M_ID,">");
 //Mesage ID
 sprintf(Buffer,"Message-ID: %s\r\n",M_ID);
 Cf.Write(Buffer,strlen(Buffer));		
 //New Article?
 if(!NArt)
   {//No the it has refernces
 	//create references field	+ References + ref-messageID
 	sprintf(Buffer,"References: %s%s\r\n",Article.ReferenceID,Article.MessageID);
 	//Write it
    Cf.Write(Buffer,strlen(Buffer));
  	}	
    if(XNA) Cf.Write("X-No-Archive: yes\r\n",19);
    //User Agent
    sprintf(Buffer,"X-Newsreader: %s\r\n",Bnews->BNewsVer);
    Cf.Write(Buffer,strlen(Buffer));
    //end of header! one Blank Line!
    Cf.Write("\r\n",2);
    //allocate extra memeory for the signature
	if(AdSig) d=406;
	 else d=2;
	//if theres a binary to add add it's size to memory needed
    if(SendBinary)
      {
       Bf = BFile("/boot/home/bnews/BBinary",B_READ_ONLY);
       Bf.GetSize(&size);
       d+=size;   
      } else size=0;
	//******-Add the Body-*******
	if((memoryalloc=(char*)malloc((i=(BodyView->TextLength())+d)))==NULL) 
	  {//Fail not enough memory
	   Cf.Unset();
	   return;
	   }
 	//set pointer to the stat of the buffer  
	cp=memoryalloc;
	//get the message Text it NULL terminates it
	BodyView->GetText(0,(i-d),cp);
	if(AdSig)
	  {//add sig
	   strcat(cp,Sig);
	   cp+=(i-d)+strlen(Sig);    
        //Keep i true because less than 404 hasn't used all of it's allocation
        i-=404-strlen(Sig);
	    }//cp mod for Binary append 
	     else cp+=i-d;
	//add the inline binary to the buffer     
	if(SendBinary)
	  {
	   BEntry Entry("/boot/home/bnews/BBinary");
	   //Add Binary
	   Bf.Read(cp,size);
	   Bf.Unset();
	   SendBinary=false;
	   //unlink the BBinary file
	   Entry.Remove();
	  }
	//set pointer to the stat of the buffer  
	cp=memoryalloc;            
    while(true)
	    {
	    //find end of the line
	    if((tp=strchr(cp,'\n'))==NULL) break;
	    //make it a string
	    *tp=(char)NULL;
	    //make it a valid UNIX line 
	    //and any line starting with a '.' must have one added so it's not mistake for end of article
	    if(*cp=='.') sprintf(Buffer,".%s\r\n",cp);
	    else         sprintf(Buffer,"%s\r\n",cp);
	    //Write it to disk
	    Cf.Write(Buffer,strlen(Buffer));
	    //incremnt the pointers
	    tp++;
	    cp=tp;
	     }
	//caculate end + 1
	tp+=i;
	if(cp!=tp)
	  { //for the last line if there is no \n
	  	if(*cp=='.') sprintf(Buffer,".%s\r\n",cp);
	    else         sprintf(Buffer,"%s\r\n",cp);
	    //Write it to disk
	    Cf.Write(Buffer,strlen(Buffer));
	  }    
	 //v.important free the memory   *** 
   free((void*)memoryalloc);   
   //end of article
   Cf.Write(".\r\n",3);    
   //seek the start of the file
   Cf.Seek(0,SEEK_SET);
   //get its size
   Cf.GetSize(&size); 	   
   //allocate a buffer
  if((memoryalloc=(char*)malloc(size+1))==NULL) 
	  {//Fail not enough memory
	   Cf.Unset();
	   return;
	  }
   //Read the article into memory	  
	Cf.Read(memoryalloc,size); 
    //close the file
    Cf.Unset();
   //Ask the server if it will accept the article 
   if(!Bnews->NCommand("POST")) goto BADEND;
   //Command was ok so
   //340 send article to be posted. 
   if(LastResponse!=340) goto BADEND;
   //send article to the server
   (void)send(Socket, memoryalloc, size, 0);
    //get the response code
   if( Bnews->NLineReturn()<3) goto BADEND;//repsonse code is 3 digits
    else LastResponse = atoi(Bnews->NTTP_Line);
   Bnews->Ns->NSStat(1, Bnews->NTTP_resp(LastResponse));
    //v.important free the memory   *** 
BADEND:free((void*)memoryalloc); 
//I like keeping what I've sent, but it may raise privacy/security issues for some
//so form version 0.9 (Possessed) Beta you can turn it off
if(Bnews->SaveSent)
  {//create sent folder if it does not exist
   create_directory("/boot/home/bnews/sent/",0777);
   RETRY: 
   sprintf(Buffer,"bnSent-%i",number);
   if(dirs.Contains(Buffer, B_ANY_NODE))
	 {//get the first free filename name-number
	  number++;
	  goto RETRY;
	  } else
	      {//make a full path
	 	   strcat(NewFile,Buffer);
		   //copy the file form current article	      
		   entry.Rename(NewFile, true);
	       }
	//add the file info to the index       
	Bnews->SKAdd(1,Subject->Text(),Uname->Text(),Buffer);      
    return;
    } else entry.Remove(); //else just delete sent!!
    return;
}

void BeNewPost::MessageReceived(BMessage *message)
{
	switch (message->what)
	 {
		case B_GED_REQUESTED: 	Ged=!Ged; //switches group editable bool=!bool :)
								Groups->SetEnabled(Ged);
								break;
		case B_PBIN_REQUESTED:  SendBinary=true;
								Send->SetEnabled(false);
								SBin->SetEnabled(false);
								if(strlen(Subject->Text())<1)
 								 {//No subject don't post
   								  Subject->MakeFocus(true);
   								  Send->SetEnabled(true);
   								  SBin->SetEnabled(true);
   								  SendBinary=false;
							      } else 
							       	 if(strlen(Groups->Text())<1)
 								 		{//No group don't post
   								  		 Groups->MakeFocus(true);
   								  	     Send->SetEnabled(true);
   								  	     SBin->SetEnabled(true);
   								  	     SendBinary=false;
							             } else	PostMessage(B_OPENFILE);
								break;
		case B_POST_REQUESTED:	Send->SetEnabled(false);
								SBin->SetEnabled(false);
								if(strlen(Subject->Text())<1)
 								 {//No subject don't post
   								  Subject->MakeFocus(true);
   								  Send->SetEnabled(true);
   								  SBin->SetEnabled(true);
   								  SendBinary=false;
							      } else 
							       	 if(strlen(Groups->Text())<1)
 								 		{//No group don't post
   								  		 Groups->MakeFocus(true);
   								  	     Send->SetEnabled(true);
   								  	     SBin->SetEnabled(true);
   								  	     SendBinary=false;
							             } else
							      			if(BodyView->TextLength()<1 & SendBinary==false)
    										  {//No body text, empty article don't post
  	 											BodyView->MakeFocus(true);
     											Send->SetEnabled(true);
     											SBin->SetEnabled(true);
								       			} else
   													{//it has a subject and a Body etc
											 		 PostNewArticle();
											 		 if(modalwin!=NULL)
											 		   {
					 				 		   		    modalwin->PostMessage(B_QUIT_REQUESTED);
		 												this->Show();
											 		   }
											 		 if(!NArt) Bread->Show();
											 		 PostMessage(B_QUIT_REQUESTED);
											         }
								break;
		case B_QRES_REQUESTED:	if(!NArt) Bread->Show();
								PostMessage(B_QUIT_REQUESTED);
								break;								
		case B_XNA_REQUESTED:	XNA=!XNA;
								break;
		case B_ADS_REQUESTED:   AdSig=!AdSig;
								break;						
		case B_SIG_REQUESTED:	(new EditSig())->Show();					
								break;		
		case B_CHECK_REQUESTED:(new BeSpellcheck(this))->Show();
								break;
		case B_OPENFILE:
	    {
	     this->Hide();
	     BMessenger msgr(this);
		 FileDialog=new BFilePanel(B_OPEN_PANEL,(BMessenger*)&msgr,(entry_ref*)NULL,0,false,
		 (BMessage *)NULL,(BRefFilter *)NULL,false,true);
		 FileDialog->Window()->SetTitle("BNews: Attach file.");
		 FileDialog->Show();
         break;
          }
	    case B_REFS_RECEIVED:
	    {//gets the returned file refs from OpenFile
	     message->FindRef("refs", &fp_ref);
		 modalwin = new ModalWait();
		 modalwin->Show();
		 if(FileDialog!=NULL) 
		  {
		   delete FileDialog; 
		   FileDialog = NULL;
		   }
		 UUCode();
		 PostMessage(B_POST_REQUESTED);
		 break;
		 }
		case B_CANCEL:
		 if(FileDialog!=NULL) 
		  {
		   delete FileDialog; 
		   FileDialog = NULL;
		   }
   		 this->Show();
   	     //As the user has cancel the attach, go back to composition mode 
   	     //with both send options.
   	     Send->SetEnabled(true);
   	     SBin->SetEnabled(true);
	     break; 			
        case B_MROT13_REQUEST: 
         (new BeRot13::BeRot13())->Show();
	     break;     
 		
		default:BWindow::MessageReceived(message);
		break;
  }
}


//Edit sig.
void EditSig::MessageReceived(BMessage *message)
{
	switch(message->what)
	 {
	 	case B_SIG_REQUESTED: SaveSig();
	 	     PostMessage(B_QUIT_REQUESTED);
	 	     break;
	 	default: BWindow::MessageReceived(message);
		break;
      }
}

//-------------------------------------------------------------------//
//  			 Edit the signature file                             //
//-------------------------------------------------------------------//
 EditSig::EditSig()
	: BWindow(BRect(25, 60, 615, 228), "BNews-editsig", B_MODAL_WINDOW,
		B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS)
{	
    register char *cp,*tp;
   	BDirectory dir("/boot/home/bnews");
	BFile Cf;
    off_t size;
    char Sig[405];
    int i;
    BFont tfont(be_fixed_font);
    rgb_color rgb;
    rgb_color rgb2;
    float XOrigin,YOrigin;
    BBox *B2;
    BStringView *SigDel;
    //Make the RGB values for the text colour
	rgb.red=20;
	rgb.green=20;
	rgb.blue=150;
	rgb.alpha=255;
	rgb2.red=0;
	rgb2.green=0;
	rgb2.blue=0;
	rgb2.alpha=255;
    //calculate window size
	XOrigin=Bounds().right/2;
	YOrigin=Bounds().bottom/2;
	background = new BView(Bounds(), "view", B_FOLLOW_ALL, B_WILL_DRAW);
	//Set Background to BLACK
	background->SetViewColor(200, 200, 200, 255);
	//add 'background'
	AddChild(background);
    //add sig delimiter
    SigDel = new BStringView(BRect(4,0,110,18), "Signature Delimter", "--",B_FOLLOW_ALL); 
    SigDel->SetHighColor(rgb);
   	background->AddChild(SigDel);
 	//add the view for body
 	BodyView = new BTextView(BRect(5,20,571,115), "Body",BRect(6,0,569,95),&tfont, &rgb2 ,B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
 	background->AddChild(new BScrollView("Body", BodyView, B_FOLLOW_LEFT | B_FOLLOW_TOP, 0, true, true));
 	BodyView->SetStylable(true); 
    BodyView->SetMaxBytes(400);
    //Fills those nasty holes left in the scroll bars!
    B2= new BBox(BRect(571,115,586,130), NULL, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE_JUMP,B_NO_BORDER);
    background->AddChild(B2);
    B2->SetViewColor(191, 191, 191, 255);
	Quit = new BButton(BRect(435,138,585,164),"QButton","Abort - Close Don't Save.",new BMessage(B_QUIT_REQUESTED));
    Esig = new BButton(BRect(5,138,155,164),"NButton","Save and Exit.",new BMessage(B_SIG_REQUESTED));
  	//Add the buttons
	background->AddChild(Esig);
  	background->AddChild(Quit);
    //load current sig. if any

   if(dir.Contains("Sigfile", B_ANY_NODE))
     {
      Cf= BFile("/boot/home/bnews/Sigfile", B_READ_WRITE);
      Cf.GetSize(&size);
      //Stop trolls and idiots trying to create large signatures
      // with a size check and line count!
      if(size<401)
        {
         Cf.Read(Sig,size);
         tp=Sig;
         i=0;
         //move cp to the last newline or the fifth
         while(i<5)
             {
              if((tp=strchr(tp,'\n'))==NULL) break;
              cp=++tp;
              i++;
               }
         if(cp!=NULL) *cp=(char)NULL;
         //add up to 5 lines
          BodyView->SetText(Sig, strlen(Sig), NULL); 
         }
    Cf.Unset(); 
   }

}

//-------------------------------------------------------------------//
//  			 Perform Spell Check                                 //
//-------------------------------------------------------------------//
 BeSpellcheck::BeSpellcheck(BeNewPost* Bp)
	: BWindow(BRect(310, 190, 560, 410), "BNews-spellcheck", B_MODAL_WINDOW,
		B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS)
{	
    BFile Cf;
    BAlert* Alert;
    BDirectory dir("/boot/home/bnews");
 
	//set reference to BeNewPost
    BPost=Bp;
    //manual lookup variable
    lookup=false;
    //position in article
    pos=0;
    //Load dictonary into memory
      if(dir.Contains("lexicon", B_ANY_NODE))
     	{
     	//Allocate memory of message
     	if((memoryalloc2=(char*)malloc((buffersize=(Bp->BodyView->TextLength()+10))))==NULL) 
	      {//Fail not enough memory
	      PostMessage(B_QUIT_REQUESTED);
	      return;
	      }
		 //Set the fp & xp pointer to the start of the buffer
	     fp = xp = (char*)memoryalloc2;
    	//get the message Text it NULL terminates it
		 Bp->BodyView->GetText(0 , buffersize, fp);
    	 Cf= BFile("/boot/home/bnews/lexicon",  B_READ_ONLY);
		//Lets get the article into memory!
		 Cf.GetSize(&size);
		 if((memoryalloc=(char*)malloc(size+2))==NULL) 
		   {//free the buffr already allocated
		    free(memoryalloc2);
		    //Fail not enough memory
	        Cf.Unset();
	        Alert = new BAlert("Memory!","Sorry, There was insufficant free memory to run spell checking.","Done",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT); 
            Alert->SetShortcut(0, B_ESCAPE); 
            Alert->Go();
            PostMessage(B_QUIT_REQUESTED); 
            return;   	    
	        }
	       
		     Cf.Read((char*)memoryalloc,size);
 	         Cf.Unset();
			 //make the screen
			 Common();
            } else {//Fail no lexicon
	             	 Cf.Unset();
	             	 Alert = new BAlert("No Dictionary!","Sorry, the lexicon file was not found.","Done",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT); 
                 	 Alert->SetShortcut(0, B_ESCAPE); 
                 	 Alert->Go();
                 	 PostMessage(B_QUIT_REQUESTED);    	    
	                 }

/*If all as gone well
we have the article to be checked is in buffer memoryalloc2 and 
char* pointer fp is set to it's start off_t	buffersize is it's size
memoryalloc points to the lexicon */

}

//-------------------------------------------------------------------//
//  			 Perform Spell Check                                 //
//-------------------------------------------------------------------//
 BeSpellcheck::BeSpellcheck(BeNewMail* Bp)
	: BWindow(BRect(310, 190, 560, 410), "BNews-spellcheck", B_MODAL_WINDOW,
		B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS)
{	
    BFile Cf;
    BAlert* Alert;
    BDirectory dir("/boot/home/bnews");
   
	//set reference to BeNewPost
	BPost=NULL;
    BPost1=Bp;
    //manual lookup variable
    lookup=false;
    //position in article
    pos=0;
    //Load dictonary into memory
      if(dir.Contains("lexicon", B_ANY_NODE))
     	{
     	//Allocate memory of message
     	if((memoryalloc2=(char*)malloc((buffersize=(Bp->BodyView->TextLength()+10))))==NULL) 
	      {//Fail not enough memory
	      PostMessage(B_QUIT_REQUESTED);
	      return;
	      }
		 //Set the fp & xp pointer to the start of the buffer
	     fp = xp = (char*)memoryalloc2;
    	//get the message Text it NULL terminates it
		 Bp->BodyView->GetText(0 , buffersize, fp);
    	 Cf= BFile("/boot/home/bnews/lexicon",  B_READ_ONLY);
		//Lets get the article into memory!
		 Cf.GetSize(&size);
		 if((memoryalloc=(char*)malloc(size+2))==NULL) 
		   {//free the buffr already allocated
		    free(memoryalloc2);
		    //Fail not enough memory
	        Cf.Unset();
	        Alert = new BAlert("Memory!","Sorry, There was insufficant free memory to run spell checking.","Done",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT); 
            Alert->SetShortcut(0, B_ESCAPE); 
            Alert->Go();
            PostMessage(B_QUIT_REQUESTED); 
            return;   	    
	        }
	       
		     Cf.Read((char*)memoryalloc,size);
		     Cf.Unset();
             //make the screen
             Common();
            } else {//Fail no lexicon
	             	 Cf.Unset();
	             	 Alert = new BAlert("No Dictionary!","Sorry, the lexicon file was not found.","Done",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT); 
                 	 Alert->SetShortcut(0, B_ESCAPE); 
                 	 Alert->Go();
                 	 PostMessage(B_QUIT_REQUESTED);    	    
	                 }
}

void  BeSpellcheck::Common(void)
{
 BStringView *Otitle,*Title; 
 rgb_color rgb;
 rgb_color rgb2;
 float XOrigin,YOrigin;
 BBox *B2;
    
	  //This is where the build window goes...
       //Make the RGB values for the text colour
	   rgb.red=20;
	   rgb.green=20;
	   rgb.blue=150;
	   rgb.alpha=255;
	   rgb2.red=160;
	   rgb2.green=0;
	   rgb2.blue=0;
	   rgb2.alpha=255;
       //calculate window size
	   XOrigin=Bounds().right/2;
	   YOrigin=Bounds().bottom/2;
	   background = new BView(Bounds(), "view", B_FOLLOW_ALL, B_WILL_DRAW);
	   //Set Background to BLACK
	   background->SetViewColor(200, 200, 200, 255);
	  //add 'background'
	   AddChild(background);
 	  //add the view for body
 	   Title = new BStringView(BRect(4,4,110,18), "S", "Spell Checker.",B_FOLLOW_ALL); 
       Title->SetHighColor(rgb);
 	   QWord = new BTextControl(BRect(6,30,230,55),"QWord","Query","",0,B_FOLLOW_ALL, B_WILL_DRAW);
 	   QWord->SetDivider(45);
 	   QWord->SetHighColor(rgb2);
 	   QWord->SetEnabled(true);
 	   RWord = new BTextControl(BRect(6,60,230,80),"RWord","Suggests","",0,B_FOLLOW_ALL, B_WILL_DRAW);
 	   RWord->SetDivider(45);
 	   RWord->SetEnabled(false);
 	   Others = new BListView(BRect(55,90,215,170), "Body", B_SINGLE_SELECTION_LIST);//&tfont, &rgb2 ,B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
 	   Others->SetSelectionMessage(new BMessage(B_WORD_CHANGE));
 	   B2= new BBox(BRect(215,170,230,185), NULL, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE_JUMP,B_NO_BORDER);
       Others->SetHighColor(rgb);
       Otitle = new BStringView(BRect(6,90,30,105), "Or", "Or",B_FOLLOW_ALL); 
       Otitle->SetHighColor(rgb);
  	   Replace = new BButton(BRect(5,196,53,206),"RButton","Replace",new BMessage(B_REPLACE_REQUESTED));
	   Ignore = new BButton(BRect(53,196,101,206),"IButton","Ignore",new BMessage(B_SCHK_REQUESTED));
   	   Add = new BButton(BRect(101,196,149,206),"AButton","Add",new BMessage(B_ADDLEX_REQUESTED));
   	   Suggest = new BButton(BRect(149,196,197,206),"SButton","Lookup",new BMessage(B_LOOKUP_REQUESTED));
   	   Quit = new BButton(BRect(197,196,245,206),"QButton","Quit",new BMessage(B_FREEQ_REQUESTED));
   	   background->AddChild(Title); 	
 	   background->AddChild(QWord);
 	   background->AddChild(RWord);	
       background->AddChild(new BScrollView("Body", Others, B_FOLLOW_LEFT | B_FOLLOW_TOP, 0, true, true));
	   background->AddChild(B2);
	   background->AddChild(Otitle);
	   background->AddChild(Replace);    
	   background->AddChild(Ignore);
	   background->AddChild(Add);
	   background->AddChild(Suggest);
	   background->AddChild(Quit);		 
			 
/*If all as gone well
we have the article to be checked is in buffer memoryalloc2 and 
char* pointer fp is set to it's start off_t	buffersize is it's size
memoryalloc points to the lexicon */
PostMessage(B_SCHK_REQUESTED); 
}
void  BeSpellcheck::MessageReceived(BMessage *message)
{
 int sel;
	switch(message->what)
	 {  
	 	case B_SCHK_REQUESTED:
	   		 CheckIt();
	         break;
	    //It is vitally important that the memory is freed when the spellchecker quits
	    case B_FREEQ_REQUESTED:
	         //free the buffers already allocated
		     free(memoryalloc2);
	         free(memoryalloc);
	    	 PostMessage(B_QUIT_REQUESTED); 
	    	 break;
	    case B_LOOKUP_REQUESTED:
	         lookup=true;
	         CheckIt();
	         break;
	    case B_ADDLEX_REQUESTED:
	         AddWord();
	         break;     
	    case B_REPLACE_REQUESTED:
	         Replacer();
	         CheckIt();
	         break;    
	    case B_WORD_CHANGE:
	         if((sel=Others->CurrentSelection(0))<0) break;
   	         strcpy(lword,((const BStringItem*)Others->ItemAt(sel) )->Text());
   	         RWord->SetText(lword);
   	         break;
	    
	    default: BWindow::MessageReceived(message);
		break;
      }
}
//Add new word to lexicon
void  BeSpellcheck::AddWord(void)
{
 BAlert* Alert;
 BFile Cf;
 off_t size;
 
//copy the replacement word Because it may have beened edited
 strcpy(lword,QWord->Text());
 //We already know it exists!
 Cf= BFile("/boot/home/bnews/lexicon",  B_READ_WRITE);
 //Lets get the article into memory!
 Cf.GetSize(&size);
 Cf.Seek(size,SEEK_SET);
 //make first letter uppercase
 lword[0]=toupper(lword[0]);
 //and the new line char
 strcat(lword,"\n");
 Cf.Write(lword,strlen(lword));
 Cf.Unset();
 //FREE the old lookup buffer
 free(memoryalloc);
 //FREEd
  Cf= BFile("/boot/home/bnews/lexicon",  B_READ_ONLY);
 //Lets get the article into memory!
 Cf.GetSize(&size);
 if((memoryalloc=(char*)malloc(size+2))==NULL) 
  {//free the buffr already allocated
    free(memoryalloc2);
    //Fail not enough memory
    Cf.Unset();
    Alert = new BAlert("Memory!","Sorry, There was insufficant free memory to load new dictionary!","Done",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT); 
    Alert->SetShortcut(0, B_ESCAPE); 
    Alert->Go();
    PostMessage(B_QUIT_REQUESTED); 
    return;   	    
    }
 Cf.Read((char*)memoryalloc,size);
 Cf.Unset();

}
//Replace.
void  BeSpellcheck::Replacer(void)
{
 int rwlen;
 int spos;
 //copy the replacement word Because it may have beened edited
 strcpy(lword,RWord->Text());
 //get the length of the replacement word 
 if((rwlen=strlen(lword))<1) return;
 //points to the end of current lookup word
 pos+=fp-xp;
 //set xp=fp so we can work out the difference next time
 xp=fp;
 //points to the start
 spos=pos-qlen;
 //replace the word!
 //LOCK LOOPER choose between mail and News check!
 if(BPost!=NULL)
  {
   BPost->LockLooper();
   //Delete the original word at position pos-llen
   BPost->BodyView->Delete(spos, pos);
   BPost->BodyView->Insert(spos,lword,rwlen, NULL);
   //Done :)
   //UNLOCK LOOPER
   BPost->UnlockLooper();
   } else   {
   			  BPost1->LockLooper();
   			  //Delete the original word at position pos-llen
   			  BPost1->BodyView->Delete(spos, pos);
   			  BPost1->BodyView->Insert(spos,lword,rwlen, NULL);
   			 //Done :)
   			 //UNLOCK LOOPER
  			  BPost1->UnlockLooper();
   			 }
 //adjust for different length words
 if(rwlen>qlen) pos+=rwlen-qlen;
 else if(rwlen<qlen) pos-=qlen-rwlen;
}

//Check word.
void  BeSpellcheck::CheckIt(void)
{
register char *ap,*cp;
register unsigned int i,tx2;
register long index;
char qword[512],pword[5][512];
int llen,pweight[5]={0,0,0,0,0},weight;
bool NoA1pha;

//grey out the buttons
Add->SetEnabled(false);
Ignore->SetEnabled(false);
Suggest->SetEnabled(false);
Replace->SetEnabled(false);
//to relookup a manually edited word
if(lookup)
  {
   strcpy(qword,QWord->Text());
   if((qlen=strlen(qword))>0) goto MAINLOOP;
   else goto FAIL;
   }
//get the first word to look up
RESET: NoA1pha=true;
index=fp-(char*)memoryalloc2;
//skip spaces brackets braces and quotes etc
while(index<buffersize && !(*fp>(char)64 && *fp<(char)91) && !(*fp>(char)96 && *fp<(char)124))
     {//detect quotes.. and skip quoted line
      if((fp[0]=='\n' && fp[1]=='>') || (fp[-1]=='\n' && fp[0]=='>'))
        {
         ap=fp;
         fp++;
         fp=(char *)memchr(fp,'\n',buffersize-index);
         index+=fp-ap;
         }
      fp++;
	  index++;
	  }
	 
//quit when done.	  
if(index>=buffersize) 
  {
  PostMessage(B_QUIT_REQUESTED); 
  return;
   }	  

//get the lookup word
cp=fp;
while(index<buffersize && (*cp==(char)0x27 || (*cp>(char)64 && *cp<(char)91) || (*cp>(char)96 && *cp<(char)124)))
     {
      cp++;
	  index++;
	  }
qlen=cp-fp;
strncpy(qword,fp,qlen);
qword[qlen]=(char)NULL;
//move the fp pointer on
fp=cp;
//check it is a word not an alpha numeric sequence
for(i=0;i<(unsigned)qlen;)
   {
   //it has numbers in it, skip it!
   if(qword[i]>(char)47 && qword[i]<(char)58) goto RESET;
   //check it does contain some alpha caharcters
   if(qword[i]>(char)64 && qword[i]<(char)91) NoA1pha=false;
   else if(qword[i]>(char)96 && qword[i]<(char)124) NoA1pha=false;
   i++;    
   }
//it has no Aplha goto RESET
if(NoA1pha) goto RESET;

//point to the last char remove none aplha such as ! ? .
i--;
if((qword[i]>(char)0 && qword[i]<(char)65) ||
   (qword[i]>(char)90 && qword[i]<(char)97) ||
   (qword[i]>(char)123 && qword[i]<(char)128))  {//then short word length as well
    											 qword[i]=(char)NULL;
    											 qlen--;
    											  }
    											  
//SO lookup can still know the replace size!
oldqlen=qlen;
    											  
MAINLOOP:
//set cp to the start of the lexicon
ap=cp=(char*)memoryalloc;
index=0;
//Lookup a word.
while(index<size)
 {
 //get a word without overwriting the lookup buffer with NULLS
 if((cp=strchr(ap,'\n'))==NULL) break;
 llen=cp-ap;
 memcpy(lword,ap,llen);
 lword[llen]=(char)NULL;
 ap=++cp;
 index+=llen+1;
 //found it quit it's valid.          MATCH!
 if(llen>0 && qlen==llen && strncasecmp(qword,lword,llen)==0)
  {
   if(!lookup)
     {
      Add->SetEnabled(true);
      Ignore->SetEnabled(true);
      Suggest->SetEnabled(true);
      Replace->SetEnabled(true);
      PostMessage(B_SCHK_REQUESTED);
      return;
      } else {//perfect match on lookup
      			weight=0xffff;
      			goto LISTBUILD;
       			}
      
   } 
 //all the 'intelligent' look up code lives here
 //the more rules the better the spell check will work
 ///BASIC/
 //Reset the wieght value to 0
 weight=0;
 //you near always know the first letter so that will be correct
 //because we KNOW the lexicon is of format CAPITALlowercase Cat Dog etc
 if(toupper(qword[0])==lword[0]) weight=250;
 //you also know how most words end
 if(tolower(qword[qlen-1])==lword[llen-1]) weight+=40;

 //look for simple correlation
 if(qlen==llen)
  {
   weight+=40;   
   i=llen-1;
   } else 
     if(qlen>llen) i=llen;
              else i=qlen-1;
              
 //checks for matching letters so cat agains rat would score 20
 while(i>0) 
     {
      if(tolower(qword[i])==lword[i]) weight+=20;
      //so recieve aganist receive ie transposed
      if(tolower(qword[i])==lword[i+1] && tolower(qword[i+1])==lword[i]) weight+=8;
      i--;
     } 

LISTBUILD:  		
 //compare it with previous words
 //replace as necessary!
 for(i=0;i<5;)
   {
    if(weight>pweight[i]) 
  	  {
  	  
  	  //move all the words down one  	
  	   for(tx2=4;tx2>i;)
  	      {
  	      if(pweight[tx2-1]>0)
  	        {
  	         strcpy(pword[tx2],pword[tx2-1]);
  	         pweight[tx2]=pweight[tx2-1];
  	        }
			tx2--;
  	      }
        strcpy(pword[i],lword);
  	  	pweight[i]=weight;
  		break;
  	   }
  	i++;	
   }

 }

//Empty the list window first 
 Others->MakeEmpty();
//change case of first letter to match that of query word, then add it.
 if(isupper(qword[0]))
	{
      for(i=0;i<5;)
         {
     	  if(pweight[i]>0)
     	     {
     	      pword[i][0]=toupper(pword[i][0]);     
     	      Others->AddItem(new BStringItem(pword[i]));
     	      }
  	      i++;	
          }
	 } else
	      {
	        for(i=0;i<5;)
         	   {
     	  		if(pweight[i]>0)
     	  		  {
     	  		   pword[i][0]=tolower(pword[i][0]);     
     	  		   Others->AddItem(new BStringItem(pword[i]));
     	  		   }
  	      		i++;	
          		}
	       }

//pass all the words to the checker window
 QWord->SetText(qword);
 RWord->SetText(pword[0]);
 llen=strlen(qword);
//reset the lookup value  
FAIL: 
if(lookup) qlen=oldqlen;
lookup=false;
Add->SetEnabled(true);
Ignore->SetEnabled(true);
Suggest->SetEnabled(true);
Replace->SetEnabled(true);
}


void EditSig::SaveSig(void)
{
  register char *cp,*tp;
  BFile Cf;
  char Sig[405];
  int i;
  //Get the new Sig
  BodyView->GetText(0 , 400, Sig);
  //Check there is text
  if(strlen(Sig)<1) return;
  //Create Sig file
  //Delete existing file
  Cf= BFile("/boot/home/bnews/Sigfile",  B_ERASE_FILE );
  Cf.Unset();
  //Write Config file
  Cf= BFile("/boot/home/bnews/Sigfile", B_READ_WRITE | B_CREATE_FILE);
  tp=Sig;
  i=0;
  //move cp to the last newline or the fifth
  while(i<5)
       {
        if((tp=strchr(tp,'\n'))==NULL) break;
        //++tp because we want to retain the last /n
        cp=++tp;
        i++;
         }
       if(cp!=NULL) *cp=(char)NULL;
 //Write up to 5 lines
 Cf.Write(Sig,strlen(Sig));
 Cf.Unset();
 return;    
}




//-------------------------------------------------------------------//
//    This is the respond via EMail code, just a simple mailer       //
//-------------------------------------------------------------------//
BeNewMail::BeNewMail(BeNewsWin* bn,bool NotNew)
	: BWindow(BRect(25, 60, 615, 470), "BNews-Mailer", B_MODAL_WINDOW,
		B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS)
{	
    BDirectory dir("/boot/home/bnews");
    BFile Cf;
    register char *cp,*tp;
    char *xp,*memoryalloc;
    char Newsgroups[256],Buffer[2050];
    off_t size;
    BFont tfont(be_fixed_font);
    rgb_color rgb;
    rgb_color rgb2;
    float XOrigin,YOrigin;
    BBox *B2;

	Bnews=bn;
	//Make the RGB values for the text colour
	rgb.red=20;
	rgb.green=20;
	rgb.blue=150;
	rgb.alpha=255;
	rgb2.red=0;
	rgb2.green=0;
	rgb2.blue=0;
	rgb2.alpha=255;

    //calculate window size
	XOrigin=Bounds().right/2;
	YOrigin=Bounds().bottom/2;
	background = new BView(Bounds(), "view", B_FOLLOW_ALL, B_WILL_DRAW);
	//Set Background to BLACK
	background->SetViewColor(200, 200, 200, 255);
	//add 'background'
	AddChild(background);
	//add the view for body
 	BodyView = new BTextView(BRect(5,65,571,365), "Body",BRect(6,0,569,300),&tfont, &rgb2 ,B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW);
 	background->AddChild(new BScrollView("Body", BodyView, B_FOLLOW_LEFT | B_FOLLOW_TOP, 0, true, true));
 	BodyView->SetStylable(true); 
    Uname = new BTextControl(BRect(6,5,285,25),"You","From","",0,B_FOLLOW_ALL, B_WILL_DRAW);
    Uname->SetDivider(40);
    //use this false to set the adSig flag
    Uname->SetEnabled(AdSig=false);
    MailTo = new BTextControl(BRect(295,5,584,25),"mailto","To","",0,B_FOLLOW_ALL, B_WILL_DRAW);
    MailTo->SetDivider(20);
    MailTo->SetEnabled(true);
    Subject= new BTextControl(BRect(6,31,584,51),"subject","Subject","",0,B_FOLLOW_ALL, B_WILL_DRAW);
    Subject->SetDivider(40);
    Subject->SetEnabled(true);
    AddSig = new BCheckBox(BRect(6,387,100,409),"EG","Add Signature",new BMessage(B_ADS_REQUESTED), B_FOLLOW_ALL, B_WILL_DRAW);
    background->AddChild(Uname);
    background->AddChild(Subject);
    background->AddChild(MailTo);
    background->AddChild(AddSig);
    Uname->SetText(Bnews->UserID);
    //Fills those nasty holes left in the scroll bars!
    B2= new BBox(BRect(572,365,586,381), NULL, B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE_JUMP,B_NO_BORDER);
    background->AddChild(B2);
    B2->SetViewColor(191, 191, 191, 255);
  	Quit = new BButton(BRect(435,383,585,409),"QButton","Abort - Close Don't Send.",new BMessage(B_QUIT_REQUESTED));
    Chek = new BButton(BRect(101,383,186,409),"PButton","Spell Check",new BMessage(B_CHECK_REQUESTED));
 	Que = new BButton(BRect(306,383,425,409),"FButton","Queue Email to send!",new BMessage(B_QUEU_REQUESTED));
 	Send = new BButton(BRect(186,383,305,409),"FButton","Send Email Now!",new BMessage(B_POST_REQUESTED));
	
	//Add the buttons
 	background->AddChild(Chek);
 	background->AddChild(Send);
 	background->AddChild(Que);
  	background->AddChild(Quit);
    Subject->SetEnabled(true);
	if(Socket==-1) Send->SetEnabled(false);
      //The article you are responing to quote and atrribute get groups etc     
      if(dir.Contains("Currentarticle", B_ANY_NODE) && NotNew)
     	{
	     Cf= BFile("/boot/home/bnews/Currentarticle", B_READ_ONLY);
		//Lets get the article into memory!
		 Cf.GetSize(&size);
		 if((memoryalloc=(char*)malloc(size+2))!=NULL)
			{//We have the memory
			 cp=tp=memoryalloc;
			 xp=cp+size;
			 //read the whole file into memory
			 Cf.Read(cp,size);
			 //make the whole thing a string...
			 cp[size]=(char)NULL;
			 //get all fields..
			 //From
			 if((tp=strstr(cp,"From: "))!=NULL)
			   {
			    if((cp=strstr(tp," \n"))!=NULL)
			      {
			       *cp=(char)NULL;
			       tp+=6;
			       //set the recipent
			       MailTo->SetText(tp);
			       strcpy(Article.Name,tp);
			       *cp=' ';
			       }
			    }
			 cp=tp=memoryalloc;
			 // Newsgroups:
			 if((tp=strstr(cp,"Newsgroups: "))!=NULL)
			   {
			    if((cp=strstr(tp," \n"))!=NULL)
			      {
			       *cp=(char)NULL;
			       tp+=12;
			       strcpy(Newsgroups,tp);
			       *cp=' ';
			       }
			    }
			 cp=tp=memoryalloc;
			 //Subject (all fields that will go in the new header must had the extra [space] removed
			 if((tp=strstr(cp,"Subject: "))!=NULL)
			   {
			    if((cp=strstr(tp," \n"))!=NULL)
			      {
			       *cp=(char)NULL;
			       tp+=9;
			       strcpy(Article.Subject,tp);
			       *cp=' ';
			       }
			    }
			 cp=tp=memoryalloc;    
			 //Date
			 if((tp=strstr(cp,"Date: "))!=NULL)
			   {
			    if((cp=strstr(tp," \n"))!=NULL)
			      {
			       *cp=(char)NULL;
			       tp+=6;
			       strcpy(Article.Date,tp);
			       *cp=' ';
			       }
			    }
			 cp=tp=memoryalloc;
			 //Message ID
			 if((tp=strstr(cp,"Message-ID: "))!=NULL)
			   {
			    if((cp=strstr(tp," \n"))!=NULL)
			      {
			       *cp=(char)NULL;
			       tp+=12;
			       strcpy(Article.MessageID,tp);
			       *cp=' ';
			       }
			     }
			 cp=tp=memoryalloc;
			 //References
			 if((tp=strstr(cp,"References: "))!=NULL)
			   {
			    if((cp=strstr(tp," \n"))!=NULL)
			      {
			       *cp=(char)NULL;
			       tp+=12;
			       strcpy(Article.ReferenceID,tp);
			       *cp=' ';
			       }
			    }
			 cp=tp=memoryalloc;
			 
			 //create the Quote line the first line
             sprintf(Buffer,"On %s, %s wrote in News:\n\n",Article.Date,Article.Name);
            
             if(strlen(Buffer)>79)
              {//split a line greater than on line at the space nearest the end
              unsigned int i;
              for(i=79;i>0;)
                 {
                  if(Buffer[i]==' ')
                    {
                     Buffer[i]='\n';
                     break;
                    } 
                  i--;                     
                 }
               }
			 BodyView->SetText(Buffer);
			 //free that memory
			 size=strlen(Buffer);
			 if((cp=strstr(tp,"\n. \n"))!=NULL)   
			   {
			    cp+=4;
			    while(tp<xp)
			     {
			      if((tp=strstr(cp," \n"))==NULL) break;
			      *tp='\n';
			      tp++;
			      *tp=(char)NULL;
			      sprintf(Buffer,">%s",cp);
				  BodyView->Insert((int32)size, Buffer, strlen(Buffer),NULL);
				  size+=strlen(Buffer);
				  tp++;
				  cp=tp;
			      }
			   	   
			   }	
			  //FREE up that memory!!!!! 
             free((void*)memoryalloc);
             //Write the groups
           //  Groups->SetText(Newsgroups);
             //MUST be 'Re: ' not 'RE: ' or any other nonesense
             //such as 'Re(2): ', 'Re:' or 'Sv: '
             cp=Article.Subject;
             if(cp[0]=='R' && cp[1]=='e' && cp[2]==':' && cp[3]==' ') strcpy(Buffer,Article.Subject);
             else //almost right just screwy case
             if((cp[0]=='R' || cp[0]=='r') && (cp[1]=='E' || cp[1]=='e') && cp[2]==':' && cp[3]==' ')
                {
                cp[0]='R';
                cp[1]='e';
                strcpy(Buffer,Article.Subject);
                }  
             else //some Re: and no space
             if((cp[0]=='R' || cp[0]=='r') && (cp[1]=='E' || cp[1]=='e') && cp[2]==':' && cp[3]!=' ')
                {
                cp+=3;
                strcpy(Buffer,"Re: ");
                strcat(Buffer,cp);
                }
             else //'Sv: ' never seen it but it's mentioned in a RFC as legacy so
             if(cp[0]=='S' && cp[1]=='v' && cp[2]==':' && cp[3]==' ')
               {
                cp[0]='R';
                cp[1]='e';
                strcpy(Buffer,Article.Subject);
                } 
             else//'Re(2): ' never seen it but it's mentioned in a RFC as legacy so
             if(cp[0]=='R' && cp[1]=='e' && cp[2]=='(' && cp[3]!='2' && cp[4]=='2' && cp[5]!=')' && cp[6]==':' && cp[7]==' ')
                {
                cp+=7;
                strcpy(Buffer,"Re: ");
                strcat(Buffer,cp);
                }   
             else  
                 {
                  strcpy(Buffer,"Re: ");
                  strcat(Buffer,cp);
                  }  

             Subject->SetText(Buffer);   
			}	 
		} //can't get artilce to respone to quit!   
      

}

void BeNewMail::MessageReceived(BMessage *message)
{
	switch(message->what)
	 {
	    case B_QUEU_REQUESTED:
	    	 SendNow=false;
	    	 
	    	 Send->SetEnabled(false);
	    	 Que->SetEnabled(false);
	    	 if(strlen(Subject->Text())<1)
 			   {//No subject don't post
   			    Subject->MakeFocus(true);
   			    if(Socket==-1) Send->SetEnabled(false);
   			    Que->SetEnabled(true);
			    } else 
			    	 if(strlen(MailTo->Text())<1)
 			    	   {//No group don't post
   						MailTo->MakeFocus(true);
   						if(Socket==-1) Send->SetEnabled(false);
   						Que->SetEnabled(true);
						} else
							if(BodyView->TextLength()<1)
    						  {//No body text, empty article don't post
  	 					       BodyView->MakeFocus(true);
     						   if(Socket==-1) Send->SetEnabled(false);
     						   Que->SetEnabled(true);
							   } else
   								  {//it has a subject and a Body etc
	    	 					   SendMail();
	    	 					   PostMessage(B_QUIT_REQUESTED);
			     			       }
	
	 	     break;
	    case B_POST_REQUESTED:
	    	 SendNow=true;
	      	 Send->SetEnabled(false);
	    	 Que->SetEnabled(false);
			 if(strlen(Subject->Text())<1)
 			   {//No subject don't post
   			    Subject->MakeFocus(true);
   			    if(Socket==-1) Send->SetEnabled(false);
   			    Que->SetEnabled(true);
			    } else 
			    	 if(strlen(MailTo->Text())<1)
 			    	   {//No group don't post
   						MailTo->MakeFocus(true);
   						if(Socket==-1) Send->SetEnabled(false);
   						Que->SetEnabled(true);
						} else
							if(BodyView->TextLength()<1)
    						  {//No body text, empty article don't post
  	 					       BodyView->MakeFocus(true);
     						   if(Socket==-1) Send->SetEnabled(false);
     						   Que->SetEnabled(true);
							   } else
   								  {//it has a subject and a Body etc
	    	 					   SendMail();
	    	 					   PostMessage(B_QUIT_REQUESTED);
			     			       }
	 	     break;
	 	case B_ADS_REQUESTED:   AdSig=!AdSig;
								break;
		case B_CHECK_REQUESTED:(new BeSpellcheck(this))->Show();
		                       break;								     
	 	default: BWindow::MessageReceived(message);
		break;
      }
}

void BeNewMail::SendMail(void)
{

 BDirectory dir("/boot/home/bnews");
 BFile Cf;
 BMailMessage *mail;
 unsigned int i,d;
 off_t size;
 char *cp,*tp,*memoryalloc,Sig[405],Buffer[2000];
 BAlert *Alert; 

//get sig
	if(AdSig)
	  {
	   d=404;
	   i=0;
	   if(dir.Contains("Sigfile", B_ANY_NODE))
     	 {
          Cf= BFile("/boot/home/bnews/Sigfile", B_READ_WRITE);
          Cf.GetSize(&size);
	      if(size<401)
            {
			 tp=Sig;
             tp+=4;
             strcpy(Sig,"\n--\n");
             Cf.Read(tp,size);
             while(i<5)
                 {
                  if((tp=strchr(tp,'\n'))==NULL) break;
                  cp=++tp;
                  i++;
                  }
             if(cp!=NULL) *cp=(char)NULL;
             }
           Cf.Unset(); 
           }      
	     }	 else d=2;
	//******-Add the Body-*******
	if((memoryalloc=(char*)malloc((i=(BodyView->TextLength())+d)))==NULL) 
	  {//Fail not enough memory
	   Cf.Unset();
	    Alert = new BAlert("Memory!","Sorry, There was insufficant free memory to construct this email message.","Done",NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT); 
        Alert->SetShortcut(0, B_ESCAPE); 
        Alert->Go();
    	return;
	   }
  	//set pointer to the stat of the buffer  
	cp=memoryalloc;
	//get the message Text it NULL terminates it
	BodyView->GetText(0 , (i-d), cp);
	if(AdSig) strcat(cp,Sig);    

 //Construct new email meassge
 mail =  new BMailMessage();
 //Get recipent
 strcpy(Buffer,MailTo->Text());
 mail->AddHeaderField(B_MAIL_TO,Buffer);
 //get subject
 strcpy(Buffer,Subject->Text());
 mail->AddHeaderField(B_MAIL_SUBJECT,Buffer);
 //add the body
 mail->AddContent(cp,strlen(cp));
 //send mail!
 //SendNow holds quue or Send flag other is delete after sent and is false
 mail->Send(SendNow,false);	
 if(SendNow) Bnews->Ns->NSStat(3, "Mail Sent.");
  else Bnews->Ns->NSStat(3, "Mail Queued.");        

 free((void*)memoryalloc); 
 return;
}

///Authentication SIMPLE (RFC2980)
bool BeNewsWin::Authentication()
{
 char Buffer[280];

 //Check for Authentication	
 if(!NCommand("AUTHINFO")) return(false);
 //Should return bad command syntax if it recognises the command
 if(LastResponse!=501) return(false);
 Ns->NSStat(2,"Indicates support.");
 //Send User name
 sprintf(Buffer, "AUTHINFO USER %s",UserName);
 if(!NCommand(Buffer)) return(false);
 //ONLY username required 281=Authenication accepted
 if(LastResponse==281) return(true);
 //If not 281 and it's not asking for more info 381 return failed
 if(LastResponse!=381) return(false);
 //Send Password SILENTLY
 //Turn off command view
 NLoud=false;
 Ns->NSStat(2,"Sending Password.");
 //Send Password
 sprintf(Buffer, "AUTHINFO PASS %s",PassWord);
 if(!NCommand(Buffer)) return(false);
 //Turn on command view
 NLoud=true;
 //Authenication failed if not 281
 if(LastResponse!=281) return(false);
   else return(true);	
}

//Decode inline binaries such as pictures
//has built in UUDECODE, auto-rename and file launch
//returns true on fail
bool BeNewsWin::BinaryDecode(char *FB)
{
BDirectory dirs("/boot/home/bnews/attached/");

char FPath[512],store;
char *memoryalloc,*ap,*cp,*ep;
char FileName[256],BackUp[256],Byte[4];
int block=0,written=0;
bool ExitStatus=true;
off_t size=0,fsize;
BFile Cf;

strcpy(FPath,"/boot/home/bnews/attached/");
Cf = BFile(FB, B_READ_ONLY);
Cf.GetSize(&size);
if((memoryalloc=(char*)malloc(size+1))==NULL)
   {
    Cf.Unset();
    return(ExitStatus);
    }
cp=memoryalloc;
ep=cp+size;
Cf.Read(cp,size);
Cf.Unset();
////////////////////////the real code/////////////
TRYAGAIN:
//Find the Start of the imbeded image, skip if begin not found..
if((ap=strstr(cp,"begin "))==NULL) goto ENDF;
//skip it
if((ap=strchr(ap,' '))==NULL) goto ENDF; 
ap++;
//the next chrs should be a number - UNIX file mode skip it...
if(atoi(ap)==0)
  {
   //maybe someone wrote begin in preceeding text
   cp=ap;
   goto TRYAGAIN;
   }
//skip the number   
if((ap=strchr(ap,' '))==NULL) goto ENDF;
ap++;
//theorically we should be left with a filename
cp=ap;
while(cp<ep)
  {
   cp++;
   if(*cp==' ' || *cp=='\n') break;
   if(*cp=='\r' || *cp=='\n') break;
   }
//store cp   
store=*cp;
*cp=(char)NULL;
if(strlen(ap)==0) goto ENDF;
strcpy(FileName,ap);
//final vaildation
cp++;
if(store==' ') if((cp=strchr(cp,'\n'))==NULL) goto ENDF;
//start of data 
cp++;
//scan date and fix BNews ' \n' line ends and get it's size
ap=memoryalloc;
//begin UUDECODE
fsize=0;
//get first line size
block=(int)*cp-32;
//set bytes written to 0
written=0;
//point to first data byte
cp++;
while(cp<ep)
   {
    if(cp[0]==' ' && cp[1]=='\n')
      {//allow for BNews space
       cp+=2;
       block=(int)*cp-32;
       written=0;
	   cp++;
       } else if(cp[0]=='\r' && cp[1]=='\n')
                  {//allow for BNews space
                   cp+=2;
                   block=(int)*cp-32;
                   written=0;
	               cp++;
                   }
    //because most encodings end in a '\n then "end" !   block==0x60-0x20
    if((cp[0]=='e' &&  cp[1]=='n' && cp[2]=='d') || block==0x40) break;
    if(*cp==0x60) Byte[0]=0;
     else         Byte[0]=*cp-32;
	cp++;
	if(*cp==0x60) Byte[1]=0;
     else         Byte[1]=*cp-32;
	cp++;
	if(*cp==0x60) Byte[2]=0;
     else         Byte[2]=*cp-32;
	cp++;
	if(*cp==0x60) Byte[3]=0;
     else         Byte[3]=*cp-32;
	//shift the first byte 2 left and add the bit 4 and 5 of the second byte
   if(written<block) 
      {
       *ap=(char)(Byte[0] << 2) | ((Byte[1] & 0x30) >> 4);
       written++;
       fsize++;
       ap++;
       } 
    //shift bits 0-3 of the second byte left 4 and add bits 3-5 of third byte after shift right 2
    if(written<block)
      {
       *ap=(char)((Byte[1] & 0xF) << 4) | ((Byte[2] & 0x3c) >> 2);
       written++;
       fsize++;
       ap++;
       }
     //shift bits 0-1 of byte three left 6 and add the forth byte
    if(written<block) 
      {
       *ap=(char)((Byte[2] & 0x3) << 6) | Byte[3];
       written++;
       fsize++;
	   ap++;
       }
    //final increament at the bottom of the loop! after writes
    cp++; 
    }
//write the file to disk
//re-use the written variable for duplicate filename handling
written=1;
strcpy(BackUp,FileName);
if(fsize)
   {//create sent folder if it does not exist
   create_directory(FPath,0777);
   RETRY: 
   if(dirs.Contains(FileName, B_ANY_NODE))
	 {//Make a new filename eg: "kate.jpg copy 2"
	   sprintf(FileName,"%s copy %i",BackUp,written++);
	   goto RETRY;
	  } else
	      {
	       BAlert *Alert;
	       //make a full path
	 	   entry_ref Eref;
		   //create full path/filename spec	 	 
	 	   strcat(FPath,FileName);
	 	   //create file	      
		   Cf= BFile(FPath, B_READ_WRITE | B_CREATE_FILE);
		   //Cf.Write(decodedata,fsize);
		   Cf.Write(memoryalloc,fsize);
		   Cf.Unset();
		   //View/play/Run it...
		   if(strlen(FileName)<20) sprintf(BackUp,"The binary has been decoded and saved as \"%s\", do you which to launch it?",FileName);
		   else  strcpy(BackUp,"The binary has been decoded and saved, do you which to launch it?");
		   Alert = new BAlert("Attachment Launch",BackUp,"Yes, Launch.","No!",NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT); 
    	   Alert->SetShortcut(0, B_ESCAPE); 
    	   int32 button_index = Alert->Go();
    	   //User said no! then return. set status to sucessful
    	   ExitStatus=false;
    	   if(button_index==1) goto ENDF;
		   //false=don't tranverse symbolic links..
	 	   BEntry entry1(FPath,true);
	 	   //get it's entry ref
	 	   entry1.GetRef(&Eref); 
		   be_roster->Launch(&Eref,(BMessage*)NULL,(team_id*)NULL);
	       }
     }
ENDF:free((void*)memoryalloc);    
return(ExitStatus);
}


//Encode UUCODE
void BeNewPost::UUCode(void)
{
char NBuffer[B_FILE_NAME_LENGTH],Buffer[B_FILE_NAME_LENGTH+100],Byte[4],EByte[5];
register int block=0,togo=0,written=0;
BEntry Entry("/boot/home/bnews/BBinary");
off_t size=0;
BFile Cf,Ef;
Binline=0;
//BDirectory	fp_dir(&fp_ref);
BEntry	fp_entry(&fp_ref,true);//fp_name);
fp_entry.GetName(NBuffer);
//Open file to encode
Cf = BFile(&fp_entry, B_READ_ONLY);
Cf.GetSize(&size);
//calculate the size required to encode
Binline=1+(size/48);
//esize=(int)(size*1.25);
//file size limit, can't send above 7900 lines! if(esize>511872)
if(Binline>7900) return; //with the blank end line it's 505408 but I am giving a litle extra 7901 lines... abs max
//unlink BBinary
Entry.Remove();
//create new file
Ef = BFile("/boot/home/bnews/BBinary",B_CREATE_FILE | B_READ_WRITE);
//Write the fisrt line "begin" + <UNIX File Mode> + <filename>
//permmissions are all read, user+group write, no execute
sprintf(Buffer,"\nbegin 664 %s\n",NBuffer);
Ef.Write(Buffer,strlen(Buffer));
//other none data lines
Binline+=5;
EByte[4]=(char)NULL;
if(size>47)
     {//encode as normal 48 bytes 48+32=(char)P
      Ef.Write("P",1);
      togo=48;
      }else {//change the bytes in original file count for line
            EByte[0]=(char)(size+0x20);
            Ef.Write(EByte,1);
   		    }
while(written<size)
  {
   for(block=0;block<togo;)
      {
       //read 3 bytes to encode
       //padding for short lines as the shortest line is 1 byte read - byte[0]
       Byte[1]=Byte[2]=0x20;
       Cf.Read(Byte,3);
       EByte[0]=(char)((Byte[0] >> 2) & 0x3F);
       EByte[1]=(char)(((Byte[1] >> 4) & 0xF) | (((Byte[0] & 0x3) << 4) & 0x30));
       EByte[2]=(char)(((Byte[2] >> 6) & 0x3) | (((Byte[1] & 0xF) << 2) & 0x3C));
       EByte[3]=(char)(Byte[2] & 0x3F);
       if(EByte[0]==(char)0) EByte[0]=0x60;
         else 		         EByte[0]+=0x20;
	   if(EByte[1]==(char)0) EByte[1]=0x60;
         else 		         EByte[1]+=0x20;
	   if(EByte[2]==(char)0) EByte[2]=0x60;
         else 		   		 EByte[2]+=0x20;
	   if(EByte[3]==(char)0) EByte[3]=0x60;
         else		   		 EByte[3]+=0x20;
	   //write the encoded data
	   Ef.Write(EByte,4);
	   block+=3;
	   }
	written+=block;
	togo=size-written;
    if(togo>47)
     {//encode as normal 48 bytes 48+32=(char)P
      Ef.Write("\nP",2);
      togo=48;
      }
      else {//change the bytes in original file count for line
            EByte[0]='\n';
            EByte[1]=(char)(togo+0x20);
            EByte[2]=(char)NULL;
            Ef.Write(EByte,2);
   		    }
   }
Ef.Write("'\nend",5);
Ef.Unset();
Cf.Unset();
}


//Modal wait window!
//dialog to watch while the binaries are encoded!! :)
ModalWait::ModalWait()
	: BWindow(BRect(200, 200, 500, 300), "BNews-wait", B_MODAL_WINDOW,
		B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS)
{
 BView *background;
 BStringView *Title1, *Title2;
 BFont zfont(be_fixed_font);
 zfont.SetSize(13.0);
 zfont.SetFace(B_BOLD_FACE);
 background = new BView(Bounds(), "mwview", B_FOLLOW_ALL, B_WILL_DRAW);
 background->SetViewColor(200, 200, 200, 255);
 AddChild(background);
 Title1 = new BStringView(BRect(10,4,158,17), "Modalwait View", BN_STATUS,B_FOLLOW_ALL); 
 Title2 = new BStringView(BRect(20,42,304,62), "Modalwait View1", BN_STATUS,B_FOLLOW_ALL); 
 Title1->SetText("Binary Enoding.");
 Title1->SetFont(&zfont);
 Title1->SetHighColor(220, 0, 0, 255);
 Title2->SetText("Please Wait... Binary Encode And Transmit In Progress.");
 Title2->SetHighColor(0, 0, 190, 255);
 background->AddChild(Title1);
 background->AddChild(Title2);

}


void ModalWait::MessageReceived(BMessage *message)
{
	switch(message->what)
	 {
	 	default: BWindow::MessageReceived(message);
		break;
      }
}


ManualFileDecode::ManualFileDecode(BeNewsWin* Bn): BWindow(BRect(200, 150, 500, 200), "BNews-wait", B_MODAL_WINDOW,
		B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS)
{
 
 BStringView *Title1;
 BFont zfont(be_fixed_font);
 BNews=Bn;
 zfont.SetSize(13.0);
 zfont.SetFace(B_BOLD_FACE);
 background = new BView(Bounds(), "mwview", B_FOLLOW_ALL, B_WILL_DRAW);
 
 background->SetViewColor(200, 200, 200, 255);
 AddChild(background);
 Title1 = new BStringView(BRect(10,4,158,17), "Manualdec View", BN_STATUS,B_FOLLOW_ALL); 
 Title2 = new BStringView(BRect(20,20,250,35), "Manualdec View1", BN_STATUS,B_FOLLOW_ALL); 
 Quit = new BButton(BRect(250,10,298,40),"QButton","Done.",new BMessage(B_QUIT_REQUESTED));
 Title1->SetText("UUDecode.");
 Title1->SetFont(&zfont);
 Title1->SetHighColor(220, 0, 0, 255);
 Title2->SetText("File?");
 Title2->SetHighColor(0, 0, 190, 255);
 background->AddChild(Title1);
 background->AddChild(Title2);
 background->AddChild(Quit); 
 PostMessage(B_OPENFILE);
}
//more multitasking.... 
int32 newthreadN1(ManualFileDecode *md)
{
 if((md->BNews)->BinaryDecode(md->Apath)) 
   {
     md->background->Window()->Lock();
     md->Title2->SetText("Sorry, UUDecode Failed!");
     md->background->Window()->Unlock();
     }else
        {
   		 md->background->Window()->Lock();
   		 md->Title2->SetText("Decode & Saved to \\home\\bnews\\attached");
  	     md->background->Window()->Unlock();
   		 }
  md->background->Window()->Lock(); 	  
  md->Quit->SetEnabled(true);
  md->background->Window()->Unlock();	
 return(0);
}

void ManualFileDecode::MessageReceived(BMessage *message)
{
	switch(message->what)
	 {
   
    	case B_OPENFILE:
	    	{
	     	this->Hide();
	     	BMessenger msgr(this);
		 	FileDialog=new BFilePanel(B_OPEN_PANEL,(BMessenger*)&msgr,(entry_ref*)NULL,0,false,
		 	(BMessage *)NULL,(BRefFilter *)NULL,false,true);
		 	FileDialog->Window()->SetTitle("BNews: Manual UUDecode.");
		 	FileDialog->Show();
         	break;
          	}
	    case B_REFS_RECEIVED:
	    	{//gets the returned file refs from OpenFile
	    	 thread_id Tx;
	    	 this->Show();
	    	 Quit->SetEnabled(false);
	         Title2->SetText("UUDecode in progress.");
	         message->FindRef("refs", &fp_ref);
	         BEntry entry(&fp_ref,true);
 		     if(FileDialog!=NULL) 
		       {
		        delete FileDialog; 
		        FileDialog = NULL;
		        }
             entry.GetPath(&path);
             if(path.Path()!=NULL) strcpy(Apath,path.Path());
              else Apath[0]=(char)NULL;
 			 Tx=spawn_thread(newthreadN1,"Decoder",75,this);
	          if(Tx) resume_thread(Tx);
		    }
 		break;
		case B_CANCEL:
		     {
		 	  if(FileDialog!=NULL) 
		  	    {
		   		 delete FileDialog; 
		   		 FileDialog = NULL;
				 }
   		 	  this->Show();
   		 	  if(strlen(Apath)<1) Title2->SetText("UUDecode canceled.");
			 }	     
	     break; 					
	 
	    default: BWindow::MessageReceived(message);
		break;
      }
}
//Crude Knitting code
void BeNewsWin::Knit()
{
char FB[]="/boot/home/bnews/kept/knitted";
BEntry Entry(FB);
BDirectory dir("/boot/home/bnews/kept");
char FF[]="/boot/home/bnews/kept/k";
char Next[255],FileName[40],*memoryalloc,*ap,*cp,*ep;
off_t size=0;
BFile Cf,Ef;
int count=0;

Entry.Remove();
//create new file
Ef = BFile(FB,B_CREATE_FILE | B_READ_WRITE);
while(true)
   {
   NOTZERO:sprintf(Next,"%s%i",FF,count);
    sprintf(FileName,"k%i",count++);
    //Break if file doesn't exist
    if(!dir.Contains(FileName, B_ANY_NODE))
      {
       //to allow files 0-x or 1-x
       if(count==1) goto NOTZERO;
       break;
       }
    //Open it
    Cf = BFile(Next, B_READ_ONLY);
    //Read it's size
    Cf.GetSize(&size);
    //Break if it's empty
    if(size==0) break;
    //Allocate memory for it
    if((memoryalloc=(char*)malloc(size+2))==NULL)
      {
       Cf.Unset();
       return;
       }
    cp=memoryalloc;
    ep=cp+size;
    ep--;
    cp[1+size]=(char)NULL;
    //read it
    Cf.Read(cp,size);
    Cf.Unset();
	//read a file into memory
	//skip any header the check protects cp from becoming NULL
	if((ap=strstr(cp,"\n. \n"))!=NULL)
	   {
	    ap+=4;
	    cp=ap;   
	    }
	 ap=cp;   
    //remove non-data lines
    while((ap=strchr(ap,'\n'))) 
       {
        ep=++ap;
        if(*ap<0x21) break;
          }
    size=ep-cp;
    if(size) Ef.Write(cp,size);
    //free up that mememory
	free((void*)memoryalloc); 
	}
Ef.Unset();	
}
/*
///////////cleaner
void BeNewsWin::BinaryDecode()
{
char FB[]="/boot/home/bnews/Currentarticle";
char FF[]="/boot/home/bnews/body";
char *memoryalloc,*ap,*cp,*ep;
off_t size=0,fsize;
BFile Cf;
Cf = BFile(FB, B_READ_ONLY);
Cf.GetSize(&size);
if((memoryalloc=(char*)malloc(size+1))==NULL)
   {
    Cf.Unset();
    return;
    }
cp=memoryalloc;
ep=cp+size;
Cf.Read(cp,size);
Cf.Unset();
ap=cp;
fsize=0;
while(ap<ep)
     {
      if(ap[0]==' ' && ap[1]=='\n') ap++;
        else if(ap[0]=='\r' && ap[1]=='\n') ap++;
      *cp=*ap;
	  ap++;
	  cp++;
	  fsize++;     
     }
//write the file to disk
//re-use the written variable for duplicate filename handling
if(fsize)
     {
       Cf= BFile(FF, B_READ_WRITE | B_CREATE_FILE);
	   //Cf.Write(decodedata,fsize);
	   Cf.Write(memoryalloc,fsize);
	   Cf.Unset();
	    }
ENDF:free((void*)memoryalloc);    
}*/